"""Optional external libraries that enhance the CMS."""

from contextlib import contextmanager

from django.conf import settings

from cms import loader


class External(object):

    """An optional external library."""

    def __init__(self, app_name):
        """Initializes the external."""
        self._app_name = app_name

    def __nonzero__(self):
        """Returns if the app is installed."""
        return self._app_name in settings.INSTALLED_APPS

    def _load(self, name):
        """
        Loads the named object from the external library.

        If the object is not present, and ImportError or an
        Attribute error will be raised.
        """
        if self:
            return loader.load_object(".".join((self._app_name, name)))
        raise ImportError

    def load_class(self, name, fallback=None):
        """
        Loads the named class from the external library.

        If the object is not present, the fallback class will
        be returned
        """
        try:
            return self._load(name)
        except (ImportError, AttributeError):
            if fallback is None:
                return object
            return fallback

    def __getattr__(self, name):
        """Loads the named class from the external library."""
        return self.load_class(name)

    def __getitem__(self, name):
        """Loads the named class from the external library."""
        return self.load_class(name)

    def load_method(self, name, fallback=None):
        """
        Loads the named method from the external library.

        If the method is not present, a no-op dummy method
        will be returned.
        """
        try:
            return self._load(name)
        except (ImportError, AttributeError):
            if fallback is None:
                return lambda *args, **kwargs: None
            return fallback

    def __call__(self, _name, *args, **kwargs):
        """
        Calls the named method in the external library.

        If the method is not present, this is a no-op.
        """
        self.load_method(_name)(*args, **kwargs)

    def context_manager(self, name, fallback=None):
        """
        Returns the named context manager from the external library.

        If the context manager is not present, a no-op dummy context
        manager will be returned.
        """
        try:
            return self._load(name)
        except (ImportError, AttributeError):
            if fallback is None:
                @contextmanager
                def context_manager(*args, **kwargs):
                    yield
                return context_manager
            return fallback


reversion = External("reversion")

watson = External("watson")

historylinks = External("historylinks")



from django.contrib import admin

from .models import APPROVED, STATUS_CHOICES


MODERATION_FIELDS = ("Moderation controls", {
    "fields": ['status']
})


class ModerationAdminBase(admin.ModelAdmin):

    fieldsets = (MODERATION_FIELDS,)

    list_filter = ['status']

    def formfield_for_choice_field(self, db_field, request, **kwargs):
        """
        Give people who have the permission to approve item an extra option to
        change the status to approved.
        """

        if db_field.name == "status":
            choices_list = STATUS_CHOICES

            # Check if the user has permission to approve this model's objects.
            if not request.user.has_perm('{}.can_approve'.format(db_field.model._meta.app_label)):
                choices_list = [x for x in STATUS_CHOICES if not x[0] == APPROVED]

            kwargs['choices'] = choices_list

        return super(ModerationAdminBase, self).formfield_for_choice_field(db_field, request, **kwargs)



from django.template.response import SimpleTemplateResponse
from django.test import TestCase, RequestFactory

from ..apps.pages.models import Country


from ..middleware import PublicationMiddleware, LocalisationMiddleware


class MiddlewareTest(TestCase):

    def setUp(self):
        self.factory = RequestFactory()
        self.request = self.factory.get('/?preview=a')

    def test_publicationmiddleware_process_request(self):
        publication_middleware = PublicationMiddleware()
        publication_middleware.process_request(self.request)

    def test_publicationmiddleware_process_response(self):
        response = SimpleTemplateResponse('../templates/pagination/pagination.html')
        publication_middleware = PublicationMiddleware()

        response = publication_middleware.process_response(self.request, response)

        self.assertEqual(response.status_code, 200)

    def test_localisationmiddleware_process_request(self):
        localisation_middleware = LocalisationMiddleware()

        self.request = self.factory.get('/media/')
        self.assertIsNone(localisation_middleware.process_request(self.request))

        self.request = self.factory.get('/admin/')
        self.assertIsNone(localisation_middleware.process_request(self.request))

        self.request = self.factory.get('/social-auth/')
        self.assertIsNone(localisation_middleware.process_request(self.request))

        self.request = self.factory.get('/gb/')
        self.assertIsNone(localisation_middleware.process_request(self.request))

        self.assertEqual(self.request.path_info, '/gb/')
        self.assertEqual(self.request.path, '/gb/')
        self.assertIsNone(self.request.country)

        # Add a country.
        obj = Country.objects.create(
            code='GB'
        )
        self.assertIsNone(localisation_middleware.process_request(self.request))
        self.assertEqual(self.request.path_info, '/')
        self.assertEqual(self.request.path, '/')
        self.assertEqual(self.request.country, obj)

    def test_localisationmiddleware_process_request_lowercase(self):
        localisation_middleware = LocalisationMiddleware()

        # Add a country.
        obj = Country.objects.create(
            code='gb'
        )

        self.request = self.factory.get('/gb/')
        self.assertIsNone(localisation_middleware.process_request(self.request))
        self.assertEqual(self.request.path_info, '/')
        self.assertEqual(self.request.path, '/')
        self.assertEqual(self.request.country, obj)

    def test_localisationmiddleware_process_response(self):
        response = SimpleTemplateResponse('../templates/pagination/pagination.html')
        localisation_middleware = LocalisationMiddleware()

        self.request = self.factory.get('/media/')
        processed_response = localisation_middleware.process_response(self.request, response)
        self.assertEqual(processed_response, response)
        self.assertEqual(processed_response.status_code, 200)

        obj = Country.objects.create(
            code='GB',
            default=False,
        )

        self.request = self.factory.get('/')

        localisation_middleware.process_request(self.request)
        processed_response = localisation_middleware.process_response(self.request, response)
        self.assertEqual(processed_response, response)

        self.assertIsNone(self.request.country)
        self.assertEqual(processed_response, response)

        # Give ourselves a direct IP.
        self.request = self.factory.get('/')
        self.request.META['REMOTE_ADDR'] = '212.58.246.103'
        localisation_middleware.process_request(self.request)
        processed_response = localisation_middleware.process_response(self.request, response)

        self.assertEqual(processed_response.status_code, 302)
        self.assertNotEqual(processed_response, response)
        self.assertEqual(self.request.country, obj)

        # Give ourselves a proxied IP.
        self.request = self.factory.get('/')
        self.request.META['HTTP_X_FORWARDED_FOR'] = '212.58.246.103'
        localisation_middleware.process_request(self.request)
        processed_response = localisation_middleware.process_response(self.request, response)

        self.assertEqual(processed_response.status_code, 302)
        self.assertNotEqual(processed_response, response)
        self.assertEqual(self.request.country, obj)

        # Give ourselves a French IP.
        self.request = self.factory.get('/')
        self.request.META['REMOTE_ADDR'] = '2.15.255.255'
        localisation_middleware.process_request(self.request)
        processed_response = localisation_middleware.process_response(self.request, response)

        self.assertEqual(processed_response.status_code, 200)
        self.assertEqual(processed_response, response)
        self.assertIsNone(self.request.country)



"""Base classes for the CMS admin interface."""

from django.contrib import admin
from django.contrib.auth.models import User
from django.contrib.admin.sites import NotRegistered

try:
    from usertools.admin import UserAdmin
except ImportError:
    from django.contrib.auth.admin import UserAdmin

from cms import externals
from cms.models.base import SearchMetaBaseSearchAdapter, PageBaseSearchAdapter
from cms.forms import CMSAdminPasswordChangeForm


class PublishedBaseAdmin(admin.ModelAdmin):

    """Base admin class for published models."""

    change_form_template = "admin/cms/publishedmodel/change_form.html"


class OnlineBaseAdmin(PublishedBaseAdmin):

    """Base admin class for OnlineModelBase instances."""

    actions = ("publish_selected", "unpublish_selected",)

    list_display = ("__str__", "is_online",)

    list_filter = ("is_online",)

    PUBLICATION_FIELDS = ("Publication", {
        "fields": ("is_online",),
        "classes": ("collapse",),
    })

    # Custom admin actions.

    def publish_selected(self, request, queryset):
        """Publishes the selected models."""
        queryset.update(is_online=True)
    publish_selected.short_description = "Place selected %(verbose_name_plural)s online"

    def unpublish_selected(self, request, queryset):
        """Unpublishes the selected models."""
        queryset.update(is_online=False)
    unpublish_selected.short_description = "Take selected %(verbose_name_plural)s offline"


class SearchMetaBaseAdmin(OnlineBaseAdmin):

    """Base admin class for SearchMetaBase models."""

    adapter_cls = SearchMetaBaseSearchAdapter

    list_display = ("__str__", "is_online",)

    SEO_FIELDS = ("Search engine optimization", {
        "fields": ("browser_title", "meta_description", "sitemap_priority", "sitemap_changefreq", "robots_index", "robots_follow", "robots_archive",),
        "classes": ("collapse",),
    })

    OPENGRAPH_FIELDS = ("Open Graph", {
        "fields": ("og_title", "og_description", "og_image"),
        "classes": ("collapse",)
    })

    OPENGRAPH_TWITTER_FIELDS = ("Twitter card", {
        "fields": ("twitter_card", "twitter_title", "twitter_description", "twitter_image"),
        "classes": ("collapse",)
    })


if externals.reversion:
    class SearchMetaBaseAdmin(SearchMetaBaseAdmin, externals.reversion["admin.VersionMetaAdmin"]):
        list_display = SearchMetaBaseAdmin.list_display + ("get_date_modified",)


if externals.watson:
    class SearchMetaBaseAdmin(SearchMetaBaseAdmin, externals.watson["admin.SearchAdmin"]):
        pass


class PageBaseAdmin(SearchMetaBaseAdmin):

    """Base admin class for PageBase models."""

    prepopulated_fields = {"slug": ("title",), }

    search_fields = ("title", "short_title", "meta_description",)

    adapter_cls = PageBaseSearchAdapter

    TITLE_FIELDS = (None, {
        "fields": ("title", "slug",),
    })

    NAVIGATION_FIELDS = ("Navigation", {
        "fields": ("short_title",),
        "classes": ("collapse",),
    })

    fieldsets = [
        TITLE_FIELDS,
        OnlineBaseAdmin.PUBLICATION_FIELDS,
        NAVIGATION_FIELDS,
        SearchMetaBaseAdmin.SEO_FIELDS,
        SearchMetaBaseAdmin.OPENGRAPH_FIELDS,
        SearchMetaBaseAdmin.OPENGRAPH_TWITTER_FIELDS
    ]


class CMSUserAdmin(UserAdmin):
    change_password_form = CMSAdminPasswordChangeForm
    invite_confirm_form = CMSAdminPasswordChangeForm

try:
    admin.site.unregister(User)
except NotRegistered:
    pass

admin.site.register(User, CMSUserAdmin)



from django.test import TestCase
from django.utils import six

from ..externals import External

from types import FunctionType
import unittest


class TestExternals(TestCase):

    def test_load(self):
        external = External('foo')

        with self.assertRaises(ImportError):
            external._load('')

    def test_load_class(self):
        external = External('foo')

        self.assertIsInstance(external.load_class(''), object)
        self.assertTrue(external.load_class('', fallback=True))

    def test_load_method(self):
        external = External('foo')
        self.assertIsNone(external.load_method('')())
        self.assertTrue(external.load_method('', fallback=True))

    @unittest.skipIf(six.PY3, "Covered by Python 2.")
    def test_context_manager(self):
        from contextlib import GeneratorContextManager

        external = External('foo')
        self.assertIs(type(external.context_manager('')), FunctionType)
        self.assertIsInstance(external.context_manager('')(), GeneratorContextManager)
        self.assertTrue(external.context_manager('', fallback=True))

        with external.context_manager('')():
            self.assertTrue(True)



from django import forms
from django.conf import settings
from django.contrib.auth.forms import PasswordChangeForm, AdminPasswordChangeForm
from django.contrib.staticfiles.storage import staticfiles_storage
from django.core.exceptions import ValidationError
from django.utils.safestring import mark_safe

from cms import debug

import json
import re
import string


class HtmlWidget(forms.Textarea):

    """A textarea that is converted into a Redactor editor."""

    def __init__(self, *args, **kwargs):
        """Initializes the HtmlWidget."""
        super(HtmlWidget, self).__init__(*args, **kwargs)

    @debug.print_exc
    def get_media(self):
        """Returns the media used by the widget."""
        js = [
            staticfiles_storage.url("cms/js/tinymce/tinymce.min.js"),
            staticfiles_storage.url("cms/js/jquery.cms.wysiwyg.js"),
        ]

        css = {}

        return forms.Media(js=js, css=css)

    media = property(
        get_media,
        doc="The media used by the widget.",
    )

    def render(self, name, value, attrs=None):
        """Renders the widget."""

        # Add on the JS initializer.
        attrs = attrs or {}
        attrs['class'] = "wysiwyg"
        attrs['data-wysiwyg-settings'] = json.dumps(
            getattr(settings, 'WYSIWYG_OPTIONS', {})
        )

        # Get the standard widget.
        html = super(HtmlWidget, self).render(name, value, attrs)

        return mark_safe(html)

# Checks a string against some rules
def password_validation(password):
    errors = []

    if len(password) < 8:
        errors.append('Your password needs to be at least 8 characters long.')

    if password.lower() == password:
        errors.append('Your password needs include at least 1 uppercase character.')

    if password.upper() == password:
        errors.append('Your password needs include at least 1 lowercase character.')

    if not re.findall(r"[\d]", password):
        errors.append('Your password needs include at least 1 number.')

    if not re.findall(r"[{}]".format(re.escape(string.punctuation)), password):
        errors.append('Your password needs include at least 1 special character.')

    return errors


class CMSPasswordChangeForm(PasswordChangeForm):

    def clean_new_password1(self):
        password = self.cleaned_data.get('new_password1')
        if password_validation(password):
            raise ValidationError([
                ValidationError(error) for error in password_validation(password)
            ])
        return password


class CMSAdminPasswordChangeForm(AdminPasswordChangeForm):

    def clean_password1(self):
        password = self.cleaned_data.get('password1')
        if password_validation(password):
            raise ValidationError([
                ValidationError(error) for error in password_validation(password)
            ])
        return password



from django.contrib.contenttypes.models import ContentType
from django.test import TestCase
from django.utils.timezone import now

from .. import externals
from ..models.managers import publication_manager
from ..apps.pages.models import ContentBase, Page

from datetime import timedelta


class TestPageContentForManagers(ContentBase):
    pass


class TestFields(TestCase):

    def _create_objects(self):
        with externals.watson.context_manager("update_index")():
            content_type = ContentType.objects.get_for_model(TestPageContentForManagers)

            self.date = now()

            self.homepage = Page.objects.create(
                title="Homepage",
                slug='homepage',
                content_type=content_type,
                publication_date=self.date + timedelta(days=10),
            )

            TestPageContentForManagers.objects.create(
                page=self.homepage,
            )

    def test_publicationmanager_select_published(self):
        self._create_objects()

        with publication_manager.select_published(True):
            self.assertEqual(Page.objects.count(), 0)

        with publication_manager.select_published(False):
            self.assertEqual(Page.objects.count(), 1)

        # We need to generate an exception within the published block.
        with self.assertRaises(TypeError), \
                publication_manager.select_published(True):
            assert 1 / 'a'



#!/usr/bin/env python
# coding: utf-8
from cms import VERSION

import platform
from setuptools import setup, find_packages


DEPENDENCIES = [
    'django',
    'Pillow',
    'sorl-thumbnail',
    'requests',
]

if platform.python_implementation() == "PyPy":
    DEPENDENCIES.append("psycopg2cffi")
else:
    DEPENDENCIES.append("psycopg2")


setup(
    name="onespacemedia-cms",
    version=".".join(str(n) for n in VERSION),
    url="https://github.com/onespacemedia/cms",
    author="Daniel Samuels",
    author_email="daniel@onespacemedia.com",
    license="BSD",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    description='CMS used by Onespacemedia',
    install_requires=DEPENDENCIES,
    extras_require={
        'usertools':  ["django-usertools"],
        'testing':  ["mock", "coverage", "coveralls", "codecov", "cookiecutter"],
        'geoip': ['geoip']
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django :: 1.7',
        'Framework :: Django :: 1.8',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0003_page_hide_from_anonymous'),
    ]

    operations = [
        migrations.RenameField(
            model_name='page',
            old_name='url_title',
            new_name='slug',
        ),
    ]
